sp = ' '
hash = '#'
bs = '\\'
tab = '\t'
nl = '\n'
cr = '\r'
x00 = '\x00'
xE4 = '\xE4'
xFF = '\xFF'
u2603 = '\u2603'  # SNOWMAN
uFFFF = '\uFFFF'
U00010905 = '\U00010905'  # PHOENICIAN LETTER WAU
U0010FFFF = '\U0010FFFF'  # Biggest valid Unicode character
var = '${non_existing}'
pipe = '|'
