window.teardownFailureOutput = {};

window.teardownFailureOutput["suite"] = [1,2,3,0,[],[0,0,15,4],[[5,6,7,0,[],[0,12,2],[],[[8,0,0,[],[0,13,0,9],[[0,10,11,0,12,13,0,0,[1,13,0],[[13,2,13]]]]],[14,0,0,[],[0,13,1,15],[[0,16,11,0,17,18,0,0,[0,14,0],[[14,5,18]]]]]],[],[2,0,2,0]]],[],[[2,16,11,0,17,0,0,0,[0,15,0,19],[[15,5,19]]]],[2,0,2,0]];

window.teardownFailureOutput["strings"] = [];

window.teardownFailureOutput["strings"] = window.teardownFailureOutput["strings"].concat(["*","*teardownFailure","*/home/peke/Devel/robotframework/utest/webcontent/spec/data/teardownFailure","*utest/webcontent/spec/data/teardownFailure","*Suite teardown failed:\nAssertionError","*PassingFailing","*/home/peke/Devel/robotframework/utest/webcontent/spec/data/teardownFailure/PassingFailing.robot","*utest/webcontent/spec/data/teardownFailure/PassingFailing.robot","*Passing","*Parent suite teardown failed:\nAssertionError","*Log","*BuiltIn","*<p>Logs the given message with the given level.\x3c/p>","*passing","*Failing","*In test\n\nAlso parent suite teardown failed:\nAssertionError","*Fail","*<p>Fails the test with the given message and optionally alters its tags.\x3c/p>","*In test","*AssertionError"]);

window.teardownFailureOutput["stats"] = [[{"elapsed":"00:00:00","fail":2,"label":"All Tests","pass":0,"skip":0}],[],[{"elapsed":"00:00:00","fail":2,"id":"s1","label":"teardownFailure","name":"teardownFailure","pass":0,"skip":0},{"elapsed":"00:00:00","fail":2,"id":"s1-s1","label":"teardownFailure.PassingFailing","name":"PassingFailing","pass":0,"skip":0}]];

window.teardownFailureOutput["errors"] = [];

window.teardownFailureOutput["baseMillis"] = 1635776179368;

window.teardownFailureOutput["generated"] = 17;

window.teardownFailureOutput["expand_keywords"] = null;

window.settings = {"background":{"fail":"DeepPink"},"logURL":"log.html","reportURL":"report.html"};

