def normal_args(a, b=None):
    pass


def normal_and_varargs(a, b=None, *varargs):
    pass


def normal_and_varargs_and_kwargs(a, b=None, *varargs, **kwargs):
    pass


def normal_and_kwargs(positional, **kwargs):
    pass


def varargs_and_kwargs(*varargs, **kwargs):
    pass


def kwargs(**kwargs):
    pass
